@@ -1,9 +1,9 @@
 """
 
-.. include:: ephysiopy/README.md
+.. include:: README.md
 
 """
-__version__ = '1.5.48'
+__version__ = '1.5.57'
 import mahotas # this is to get around a weird, possibly python3.6 related issue
 kk_path = '/home/robin/klustakwik/KlustaKwik'
 # used to bandpass filter the continuous raw openephys data when converting to
